from pyqt_svg_button import SvgButton
from pyqt_svg_label import SvgLabel
from qtpy import QtWidgets, QtCore, QtGui


class MenuBar(QtWidgets.QMenuBar):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__initVal()
        self.__initUi()

    def __initVal(self):
        self.__isHidable = False
        self.__isHiding = False
        self.__iconLbl = SvgLabel()
        self.__titleLbl = QtWidgets.QLabel()

    def __initUi(self):
        pass

    def __showMenu(self):
        self.__menuAnimation.setDirection(QtCore.QAbstractAnimation.Backward)
        self.__menuAnimation.start()
        self.__isHiding = False

    def __hideMenu(self):
        self.__menuAnimation.setDirection(QtCore.QAbstractAnimation.Forward)
        self.__menuAnimation.start()
        self.__isHiding = True

    def setTitle(self, title: str, font: QtGui.QFont = QtGui.QFont('Arial', 9)):
        self.__titleLbl.setText(title)
        self.__titleLbl.setFont(font)
        self.__titleLbl.setMinimumHeight(self.sizeHint().height())
        cornerWidget = self.cornerWidget()
        if cornerWidget:
            lay = cornerWidget.layout()
            if lay:
                lay.insertWidget(0, self.__titleLbl)
        else:
            cornerWidget = QtWidgets.QWidget()
            lay = QtWidgets.QHBoxLayout()
            lay.addWidget(self.__titleLbl)
            lay.setContentsMargins(0, 0, 2, 0)
            cornerWidget.setLayout(lay)
            self.setCornerWidget(cornerWidget, QtCore.Qt.TopRightCorner)

    def setIcon(self, icon: str):
        self.__iconLbl.setSvgFile(icon)
        self.setCornerWidget(self.__iconLbl, QtCore.Qt.TopLeftCorner)

    def getTitle(self) -> QtWidgets.QLabel:
        return self.__titleLbl

    def getIcon(self) -> SvgLabel:
        return self.__iconLbl

    def setHidable(self, flag: bool):
        self.__isHidable = flag
        self.setMouseTracking(True)
        self.setHideButton()

    def setHideButton(self):
        tool_button = self.findChild(QtWidgets.QToolButton)
        tool_button.setArrowType(QtCore.Qt.RightArrow)

        self.__showToggleBtn = SvgButton(self)
        self.__showToggleBtn.clicked.connect(self.__hideMenu)
        self.__showToggleBtn.setToolTip('Hide the menu bar')
        self.__showToggleBtn.setIcon('close.svg')

        height = self.sizeHint().height()
        # button should be square
        width = height
        self.__showToggleBtn.setFixedSize(width, height)

        cornerWidget = self.cornerWidget(QtCore.Qt.TopRightCorner)
        if cornerWidget:
            print(cornerWidget)
            # cornerWidget.layout().addWidget(self.__showToggleBtn)

        self.__menuAnimation = QtCore.QPropertyAnimation(self, b"height")
        self.__menuAnimation.valueChanged.connect(self.setFixedHeight)

        self.__menuAnimation.setStartValue(height)
        self.__menuAnimation.setDuration(200)  # default duration
        self.__menuAnimation.setEndValue(1)  # default end value

    def enterEvent(self, e):
        if self.__isHidable and self.__isHiding:
            self.__showMenu()
        return super().enterEvent(e)




from qtpy.QtCore import QSize
from qtpy.QtGui import QPalette
from qtpy.QtWidgets import QWidget, QHBoxLayout, QMainWindow
from pyqt_svg_button import SvgButton


class CornerWidget(QWidget):
    def __init__(self, base_widget=None, hint: list = ['min', 'max', 'close']):
        super().__init__()
        self.__initVal(base_widget, hint)
        self.__initUi()

    def __initVal(self, base_widget, hint):
        self._closeBtn = SvgButton(base_widget)
        self._minimizeBtn = SvgButton(base_widget)
        self._maximizeBtn = SvgButton(base_widget)

        self._fullScreenBtn = SvgButton(base_widget)
        self._fullScreenBtn.setIcon('ico/full_screen.svg')

        self._helpBtn = SvgButton(base_widget)
        self._helpBtn.setIcon('ico/help.svg')

        self._foldBtn = SvgButton(base_widget)
        self._foldBtn.setIcon('ico/fold.svg')

        self._fixBtn = SvgButton(base_widget)
        self._fixBtn.setIcon('ico/tack.svg')

        self._btn_dict = {'min': self._minimizeBtn, 'max': self._maximizeBtn, 'close': self._closeBtn,
                          'full_screen': self._fullScreenBtn, 'help': self._helpBtn, 'fold': self._foldBtn,
                          'fix': self._fixBtn}

        self._base_widget = base_widget
        self.__window = self._base_widget.window()
        self.__window.installEventFilter(self)
        self._hint = hint

    def __initUi(self):
        lay = QHBoxLayout()
        lay.setContentsMargins(0, 0, 0, 0)
        lay.setSpacing(0)

        for k in self._hint:
            # hint str
            if k in self._btn_dict:
                lay.addWidget(self._btn_dict[k])
            # widget
            else:
                lay.addWidget(k)

        self.setLayout(lay)

        self._styleInit()

        # raise - helps the button widget not to be blocked by something else
        self.raise_()

    def _styleInit(self):
        # fill the button's background with color
        if isinstance(self._base_widget, QMainWindow):
            palette = self._base_widget.menuBar().palette()
        else:
            palette = self._base_widget.palette()
        background_color = palette.color(QPalette.Base).name()
        for btn in self._btn_dict.values():
            btn.setBackground(background_color)

    def event(self, e):
        if e.type() == 100:
            self._styleInit()
        return super().event(e)

    def setMinimizedBtn(self, btn):
        self._minimizeBtn = btn

    def setMaximizedBtn(self, btn):
        self._maximizeBtn = btn

    def setCloseBtn(self, btn):
        self._closeBtn = btn

    def setFullScreenBtn(self,  btn):
        self._fullScreenBtn = btn

    def setFixBtn(self, btn):
        self._fixBtn = btn

    def setFoldBtn(self, btn):
        self._foldBtn = btn

    def setHelpBtn(self, btn):
        self._helpBtn = btn

    def getMinimizedBtn(self):
        return self._minimizeBtn

    def getMaximizedBtn(self):
        return self._maximizeBtn

    def getCloseBtn(self):
        return self._closeBtn

    def getFullScreenBtn(self):
        return self._fullScreenBtn

    def getFixBtn(self):
        return self._fixBtn

    def getFoldBtn(self):
        return self._foldBtn

    def getHelpBtn(self):
        return self._helpBtn

    def setButtonIcon(self, hint: str, icon_filename: str):
        self._btn_dict[hint].setIcon(icon_filename)

    def setButtonSize(self, w, h):
        for k, v in self._btn_dict.items():
            if v.text():
                f = v.font()
                f.setPointSize(max(11, min(5, w // 2)))
                v.setFont(f)
            elif v.icon():
                v.setIconSize(QSize(w, h))
            v.setFixedSize(w*1.25, h)



from qtpy.QtCore import Qt, QCoreApplication
from qtpy.QtGui import QGuiApplication, QFont
from qtpy.QtWidgets import QApplication
from pyqt_custom_titlebar_setter import CustomTitlebarSetter
from qt_sass_theme import QtSassTheme


# for pyqt5
def prepareQtApp():
    QCoreApplication.setAttribute(Qt.AA_UseHighDpiPixmaps)
    QGuiApplication.setHighDpiScaleFactorRoundingPolicy(Qt.HighDpiScaleFactorRoundingPolicy.PassThrough)
    QApplication.setAttribute(Qt.AA_EnableHighDpiScaling)


class QtAppManager:
    def __init__(self, w):
        self.__innerWidget = w
        self.__mainWindow = ''

    def setTheme(self, theme: str = 'dark_gray', background_darker=False):
        g = QtSassTheme()
        g.getThemeFiles(theme=theme, background_darker=background_darker)
        g.setThemeFiles(self.__innerWidget)

    def setTitleBar(self, icon_filename: str = '', font: QFont = QFont('Arial', 14),
                    hint: list = ['min', 'max', 'close'], align=Qt.AlignCenter,
                    bottom_separator: bool = False):
        self.__mainWindow = CustomTitlebarSetter.getCustomTitleBarWindow(main_window=self.__innerWidget,
                                                                        title='',
                                                                        icon_filename=icon_filename,
                                                                        font=font,
                                                                        hint=hint,
                                                                        align=align,
                                                                        bottom_separator=bottom_separator)

    def show(self):
        if self.__mainWindow:
            self.__mainWindow.show()
        else:
            self.__innerWidget.show()


from qtpy.QtWidgets import QToolBar, QWidget, QHBoxLayout, QSizePolicy, QAction, QWidgetAction
from qtpy.QtCore import Qt, QPropertyAnimation, QAbstractAnimation
from pyqt_svg_button import SvgButton


class ToolBar(QToolBar):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__initVal()
        self.__initUi()

    def __initVal(self):
        self.__foldable_flag = False
        self.__foldBtn = SvgButton()
        self.__cornerAction = QWidgetAction(self)
        self.__menuAnimation = QPropertyAnimation(self, b"height")

    def __initUi(self):
        # moving toolbar is not standard nowadays, in my opinion
        # at least I have to polish ugly looking style
        self.setMovable(False)

    def addWidget(self, widget: QWidget) -> QAction:
        self.insertWidget(self.__cornerAction, widget)
        self.__menuAnimation.setStartValue(self.sizeHint().height())

    def setFoldable(self, f: bool):
        self.__foldable_flag = f
        if self.__foldable_flag:
            self.__foldBtn.setIcon('ico/fold.svg')
            self.__foldBtn.setCheckable(True)
            self.__foldBtn.toggled.connect(self.__fold)
            self.__foldBtn.setMaximumWidth(12)

            cornerWidget = QWidget()
            lay = QHBoxLayout()
            lay.addWidget(self.__foldBtn)
            lay.setAlignment(Qt.AlignRight | Qt.AlignBottom)
            lay.setContentsMargins(0, 0, 0, 0)
            cornerWidget.setLayout(lay)
            cornerWidget.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)

            self.__cornerAction.setDefaultWidget(cornerWidget)

            self.addAction(self.__cornerAction)

            self.__menuAnimation.valueChanged.connect(self.setFixedHeight)
            self.__menuAnimation.setStartValue(self.sizeHint().height())
            self.__menuAnimation.setDuration(200)  # default duration
            self.__menuAnimation.setEndValue(self.__foldBtn.sizeHint().height())  # default end value
        else:
            # remove fold button or add tack button
            pass

    def __fold(self, f):
        if f:
            self.__menuAnimation.setDirection(QAbstractAnimation.Forward)
            self.__menuAnimation.start()
            self.__foldBtn.setIcon('ico/unfold.svg')
            self.setFixedHeight(self.__foldBtn.sizeHint().height())
        else:
            self.__menuAnimation.setDirection(QAbstractAnimation.Backward)
            self.__menuAnimation.start()
            self.__foldBtn.setIcon('ico/fold.svg')
            self.setFixedHeight(self.sizeHint().height())




from .baseWindow import FramelessWindow
from .window import CustomTitlebarWindow
from .titleBar import TopTitleBarWidget
from .menuBar import MenuBar
from .toolBar import ToolBar
from .cornerWidget import CornerWidget
from .macCornerWidget import MacCornerWidget
from .windowsCornerWidget import WindowsCornerWidget
from .icons import *


from qtpy.QtGui import QColor, QFont, qGray
from qtpy.QtCore import Qt
from qtpy.QtWidgets import QWidget, QMainWindow, QDialog, QAbstractButton, qApp
from ..widget.svgButton import SvgButton

import os, tempfile, posixpath, re
import shutil

import qtsass


class QtSassTheme:

    def __init__(self):
        # set the icons
        cur_dir = os.path.dirname(__file__)
        ico_filename = os.path.join(cur_dir, 'ico/_icons.scss')
        icon_path = cur_dir.replace(os.path.sep, posixpath.sep)
        self.__setIcoPath(ico_filename, icon_path)

    def __setIcoPath(self, ico_filename: str, icon_path: str):
        import_abspath_str = f'$icopath: \'{icon_path}/\';'
        with open(ico_filename, 'r+') as f:
            fdata = f.read()
            # if the path is still same
            if fdata.find(import_abspath_str) != -1:
                pass
            else:
                m = re.search(r'\$icopath:\s(\"|\')(.+)(\"|\')', fdata)
                # if the path was changed
                if m:
                    f.truncate(0)
                    f.seek(0, 0)
                    f.write(import_abspath_str + '\n' + '\n'.join(fdata.splitlines(True)[1:]))

                # if the path is nowhere (which means it is the first time to set the path)
                else:
                    f.seek(0, 0)
                    f.write(import_abspath_str + '\n' + fdata)

    def __setCustomThemeColor(self, var_filename: str, theme_color: str):
        with open(var_filename, 'r+') as f:
            fdata = f.read()
            m = re.search(r'\$bgcolor:(.+\n)', fdata)
            if m:
                custom_theme_code = f'$bgcolor: {theme_color};\n'
                f.truncate(0)
                f.seek(0, 0)
                f.write(custom_theme_code + ''.join(fdata.splitlines(True)[1:]))

    def __setBackgroundPolicy(self, var_filename: str, background_darker=False):
        if background_darker:
             with open(var_filename, 'r+') as f:
                fdata = f.read()
                m1 = re.search(r'\$bgcolor:(.+\n)', fdata)
                m2 = re.search(r'\$widgetcolor:(.+\n)', fdata)
                if m1 and m2:
                    w_color_v = m1.group(1)
                    bg_color_v = m2.group(1).replace('bgcolor', 'widgetcolor')
                    background_darker_code = f'$widgetcolor:{w_color_v}' \
                                             f'$bgcolor:{bg_color_v}'
                    background_darker_code += ''.join(fdata.splitlines(True)[2:])
                    f.truncate(0)
                    f.seek(0, 0)
                    f.write(background_darker_code)
        else:
            pass

    def __getStyle(self, filename):
        cur_dir = os.path.dirname(__file__)
        sass_dirname = os.path.join(cur_dir, 'sass')
        # make temporary file
        temp_file = os.path.join(tempfile.gettempdir(), os.urandom(24).hex())
        css = qtsass.compile_filename(os.path.join(sass_dirname, filename), temp_file)
        return css

    def getThemeFiles(self, theme: str = 'dark_gray', background_darker=False, output_path=os.getcwd()):
        theme_lst = ['dark_gray', 'dark_blue', 'light_gray', 'light_blue']
        cur_dir = os.path.dirname(__file__)
        official_theme_flag = theme in theme_lst
        if official_theme_flag:
            theme_prefix = theme.split('_')[0]
            ico_dirname = os.path.join(cur_dir, os.path.join('ico', theme_prefix))
            var_dirname = os.path.join(cur_dir, os.path.join(os.path.join('var', theme_prefix), theme))

        # check whether theme value is 6-digit hex color
        else:
            m = re.match(r'#[a-fA-F0-9]{6}', theme)
            if m:
                theme_color = m.group(0)
                theme_color = QColor(theme_color)

                # 'if it is, check 6-digit hex color is lighter than usual or darker')
                r, g, b = theme_color.red(), theme_color.green(), theme_color.blue()
                theme_lightness = ''
                if qGray(r, g, b) > 255 // 2:
                    theme_lightness = 'light'
                else:
                    theme_lightness = 'dark'

                # get the ico_dirname
                ico_dirname = os.path.join(cur_dir, os.path.join('ico', theme_lightness))

                # get the dark_gray/light_gray theme
                var_dirname = os.path.join(cur_dir, os.path.join(os.path.join('var', theme_lightness),
                                                                 theme_lightness+'_gray'))
            else:
                raise Exception('Invalid theme')

        sass_dirname = os.path.join(cur_dir, 'sass')
        os.chdir(output_path)
        output_dirname = 'res'
        if os.path.exists(output_dirname):
            pass
        else:
            os.mkdir(output_dirname)
        os.chdir(output_dirname)
        if os.path.exists('ico'):
            shutil.rmtree('ico')
        if os.path.exists('sass'):
            shutil.rmtree('sass')
        if os.path.exists('var'):
            shutil.rmtree('var')
        shutil.copytree(ico_dirname, 'ico')
        shutil.copytree(sass_dirname, 'sass')
        shutil.copytree(var_dirname, 'var')

        ico_filename = 'ico/_icons.scss'
        self.__setIcoPath(ico_filename, output_dirname)

        var_filename = 'var/_variables.scss'
        if official_theme_flag:
            pass
        else:
            self.__setCustomThemeColor(var_filename, theme)
        self.__setBackgroundPolicy(var_filename, background_darker)

    def setThemeFiles(self, main_window: QWidget, input_path='res', exclude_type_lst: list = []):
        if os.path.basename(os.getcwd()) != input_path:
            input_path = os.path.join(os.getcwd(), input_path)
            os.chdir(input_path)
        qtsass.compile_dirname('sass', '.')
        ico_filename = 'ico/_icons.scss'
        sass_dirname = 'sass'
        var_dirname = 'var'
        if os.path.exists(ico_filename):
            os.remove(ico_filename)
        if os.path.exists(sass_dirname):
            shutil.rmtree(sass_dirname)
        if os.path.exists(var_dirname):
            shutil.rmtree(var_dirname)

        os.chdir('../')
        if os.path.isdir(input_path):
            f_lst = ['theme.css', 'main_widget.css', 'icon_button.css', 'icon_text_button.css', 'menu_bar.css']
            f_lst = [os.path.join(input_path, f) for f in f_lst]

            with open(f_lst[0], 'r') as f:
                theme_style = f.read()
            with open(f_lst[1], 'r') as f:
                main_widget_style = f.read()
            with open(f_lst[2], 'r') as f:
                icon_button_style = f.read()
            with open(f_lst[3], 'r') as f:
                icon_text_button_style = f.read()
            with open(f_lst[4], 'r') as f:
                menu_bar_style = f.read()

            if isinstance(main_window, QMainWindow) or isinstance(main_window, QDialog):
                main_window.setStyleSheet(theme_style)
            else:
                main_window.setObjectName('mainWidget')
                main_window.setStyleSheet(theme_style + main_widget_style)

            # button
            def setButtonStyle(main_window):
                btns = main_window.findChildren(QAbstractButton)
                for btn in btns:
                    # check if text exists
                    if btn.text().strip() == '':
                        # if button type is SvgButton, let it keep its own style
                        if isinstance(btn, SvgButton):
                            pass
                        else:
                            btn.setStyleSheet(icon_button_style)  # no text - icon button style
                    else:
                        # if button type is SvgButton, let it maintain its own style
                        if isinstance(btn, SvgButton):
                            pass
                        else:
                            btn.setStyleSheet(icon_text_button_style)  # text - icon-text button style

            # check exclusion of QAbstractButton
            if QAbstractButton in exclude_type_lst:
                pass
            else:
                setButtonStyle(main_window)

            # todo check exclusion of other types
            if len(exclude_type_lst) > 0:
                for _type in exclude_type_lst:
                    widgets = main_window.findChildren(_type)
                    if isinstance(_type, QAbstractButton):
                        print(_type)

            if isinstance(main_window, QMainWindow):
                menu_bar = main_window.menuBar()  # menu bar
                menu_bar.setStyleSheet(menu_bar_style)

        # modrenize the font
        def modernizeAppFont():
            # modernize the font
            appFont = qApp.font()
            # font family: arial
            appFont.setFamily('Arial')
            # font size: 9~12
            appFont.setPointSize(min(12, max(9, appFont.pointSize() * qApp.desktop().logicalDpiX() / 96.0)))
            # font style strategy: antialiasing
            appFont.setStyleStrategy(QFont.PreferAntialias)
            qApp.setFont(appFont)
            # fade menu and tooltip
            qApp.setEffectEnabled(Qt.UI_FadeMenu, True)
            qApp.setEffectEnabled(Qt.UI_FadeTooltip, True)

        modernizeAppFont()

    def getThemeStyle(self):
        css = self.__getStyle('theme.scss')
        return css

    def getIconButtonStyle(self):
        css = self.__getStyle('icon_button.scss')
        return css

    def getIconTextButtonStyle(self):
        css = self.__getStyle('icon_text_button.scss')
        return css

    def getMenuBarStyle(self):
        css = self.__getStyle('menu_bar.scss')
        return css

    def getMainWidgetStyle(self):
        css = self.__getStyle('main_widget.scss')
        return css




from qtpy.QtCore import Qt
from qtpy.QtGui import QPalette, QFont, QColor, qGray
from qtpy.QtWidgets import QGridLayout, QWidget, QLabel, QFrame
from pyqt_svg_icon_text_widget import SvgIconTextWidget


class TopTitleBarWidget(QWidget):
    def __init__(self, base_widget: QWidget, text: str = '', font: QFont = QFont('Arial', 14),
                 icon_filename: str = None,
                 align=Qt.AlignCenter):
        super().__init__()
        self.__baseWidget = base_widget
        self.__initVal()
        self.__initUi(text=text, font=font, icon_filename=icon_filename, align=align)

    def __initVal(self):
        self.__svgIconTitleWidget = ''
        self.__iconLbl = QLabel()
        self.__titleLbl = QLabel()
        self.__cornerWidget = ''
        self.__separator = QFrame()

    def __getTitleTextColor(self, base_color):
        r, g, b = base_color.red() ^ 255, base_color.green() ^ 255, base_color.blue() ^ 255
        if qGray(r, g, b) > 255 // 2:
            text_color = QColor(255, 255, 255)
        else:
            text_color = QColor(0, 0, 0)
        return text_color.name()

    def __initUi(self, text: str, font: QFont = QFont('Arial', 14), icon_filename: str = None, align=Qt.AlignCenter):
        self.__svgIconTitleWidget = SvgIconTextWidget()

        self.__iconLbl = self.__svgIconTitleWidget.getSvgLabel()
        self.__titleLbl = self.__svgIconTitleWidget.getTextLabel()

        if icon_filename:
            self.__svgIconTitleWidget.setSvgFile(icon_filename)
        else:
            self.__iconLbl.setVisible(False)
        self.__svgIconTitleWidget.setText(text)

        self.__titleLbl.setFont(font)

        base_color = self.__baseWidget.palette().color(QPalette.Base)

        self.__titleTextColor = self.__getTitleTextColor(base_color)

        self.setStyleSheet(f'''
                            QWidget
                            {{ 
                            background-color: {base_color.name()};
                            }}
                            QLabel
                            {{
                            color: {self.__titleTextColor};
                            }}
                            '''
                           )

        self.setMinimumHeight(self.sizeHint().height())

        lay = self.__svgIconTitleWidget.layout()
        lay.setContentsMargins(0, 0, 0, 0)

        lay = QGridLayout()
        lay.addWidget(self.__svgIconTitleWidget, 0, 0, 1, 2, alignment=align)
        lay.setContentsMargins(0, 0, 0, 0)
        # todo
        # set the spacing dynamically
        lay.setSpacing(3)
        self.setLayout(lay)

    def setCornerWidget(self, cornerWidget, align=Qt.AlignRight):
        lay = self.layout()
        self.__cornerWidget = cornerWidget
        w = h = self.__titleLbl.fontMetrics().height() * 1.25
        self.__cornerWidget.setButtonSize(w, h)
        if align == Qt.AlignRight:
            lay.addWidget(self.__cornerWidget, 0, 1, 1, 1, alignment=align)
        elif align == Qt.AlignLeft:
            lay.addWidget(self.__cornerWidget, 0, 0, 1, 1, alignment=align)

    def setBottomSeparator(self):
        lay = self.layout()
        self.__separator.setFrameShape(QFrame.HLine)
        self.__separator.setFrameShadow(QFrame.Sunken)
        self.__separator.setContentsMargins(0, 0, 0, 0)
        lay.addWidget(self.__separator, 1, 0, 1, 2)

    def getIconTitleWidget(self):
        return self.__svgIconTitleWidget

    def getIcon(self):
        return self.__iconLbl

    def getTitle(self):
        return self.__titleLbl

    def getCornerWidget(self):
        return self.__cornerWidget


from qtpy.QtGui import QColor
from qtpy.QtWidgets import qApp
from cornerWidget import CornerWidget


class MacCornerWidget(CornerWidget):
    def __init__(self, base_widget=None, hint=['min', 'max', 'close']):
        super().__init__(base_widget, hint)
        self.__size = qApp.font().pointSize()
        self.__initVal()
        self.__initUi()

    def __initVal(self):
        self.__border_width = self.__size // 20
        self.__border_radius = self.__size // 2
        self.__macBtnStyle = ''

    def __initUi(self):
        self.layout().setSpacing(2)
        btns = [self._closeBtn, self._minimizeBtn, self._maximizeBtn]
        colors = ['#DD0000', '#AA8800', '#008800']
        for i in range(len(btns)):
            btn = btns[i]
            btn.setFixedSize(self.__size, self.__size)
            self.__setStyleForEachButton(btn, colors[i])

    def _styleInit(self):
        pass

    def __setStyleForEachButton(self, btn, color):
        border_color = QColor(color)
        border_color_name = border_color.name()
        background_color_name = border_color.lighter().name()

        self.__macBtnStyle = f'''
                             QPushButton
                             {{ 
                             background-color: {background_color_name};
                             border: {self.__border_width} solid {border_color_name};
                             border-radius: {self.__border_radius};
                             }}
                             '''

        btn.setStyleSheet(self.__macBtnStyle)

    def setButtonSize(self, w, h):
        self.__size = w
        self.__initVal()
        btns = [self._closeBtn, self._minimizeBtn, self._maximizeBtn]
        colors = ['#DD0000', '#AA8800', '#008800']
        for i in range(len(btns)):
            btn = btns[i]
            btn.setFixedSize(self.__size, self.__size)
            self.__setStyleForEachButton(btn, colors[i])


from qtpy.QtCore import Qt
from qtpy.QtGui import QPalette, QFont, QIcon
from qtpy.QtWidgets import QHBoxLayout, QGridLayout, QWidget, QMainWindow, QPushButton, QLabel, \
    QMenuBar, QToolButton, qApp, QSizePolicy

from baseWindow import FramelessWindow
from qtneat.window import MenuBar
from titleBar import TopTitleBarWidget
from windowsCornerWidget import WindowsCornerWidget
from macCornerWidget import MacCornerWidget


class CustomTitlebarWindow(FramelessWindow):
    def __init__(self, widget: QWidget):
        super().__init__(widget)
        self.__initVal(widget)
        self.__initUi()

    def __initVal(self, widget):
        self.__widget = widget
        self.__widget.setObjectName('mainWidget')
        self.__menubar = ''

        # if inner widget type is QMainWindow, set menu bar as navigation widget
        # navigation widget is the widget which is able to drag to move the window, double click to maximize/normal
        if isinstance(self.__widget, QMainWindow):
            self.__menubar = self.__widget.menuBar()
            self.__menubar.setObjectName('navWidget')

        self.__windowTitleIconLabel = QLabel()
        self.__titleLbl = QLabel()
        self.__iconLbl = QLabel()

        self.__topTitleBar = QWidget()
        self.__btnWidget = QWidget()
        self.__btnHint = ['min', 'max', 'close']

        self.__minimizeBtn = QPushButton()
        self.__maximizeBtn = QPushButton()
        self.__closeBtn = QPushButton()

        self.__style = qApp.platformName()
        if self.__style == 'windows' or self.__style == 'mac':
            pass
        else:
            self.__style = 'windows'

    def __initUi(self):
        self.__widget.installEventFilter(self)
        self.installEventFilter(self)

        # connect the close event with inner widget
        lay = QGridLayout()
        lay.addWidget(self.__widget)
        lay.setContentsMargins(self._margin, self._margin, self._margin, self._margin)
        lay.setSpacing(self._margin)
        self.setLayout(lay)

        self.__modernizeAppFont()

        # set frame color
        if isinstance(self.__menubar, QMenuBar):
            color = self.__menubar.palette().color(QPalette.Base)
            self.__initMenuBar()
        else:
            color = self.__widget.palette().color(QPalette.Base)
        self.setFrameColor(color)

    def __initMenuBar(self):
        self.__menubar.installEventFilter(self)
        tool_button = self.__menubar.findChild(QToolButton)
        tool_button.setArrowType(Qt.RightArrow)

    # modernize the application
    def __modernizeAppFont(self):
        # modernize the font
        appFont = qApp.font()
        # font family: arial
        appFont.setFamily('Arial')
        # font size: 9~12
        appFont.setPointSize(min(12, max(9, appFont.pointSize() * qApp.desktop().logicalDpiX()/96.0)))
        # font style strategy: antialiasing
        appFont.setStyleStrategy(QFont.PreferAntialias)
        qApp.setFont(appFont)
        # fade menu and tooltip
        qApp.setEffectEnabled(Qt.UI_FadeMenu, True)
        qApp.setEffectEnabled(Qt.UI_FadeTooltip, True)

    def eventFilter(self, obj, e) -> bool:
        if obj == self:
            # catch resize event or window state change event
            if e.type() == 14 or e.type() == 105:
                # prevent the problem that top title bar window is not visible when full screen turning off
                if e.type() == 105:
                    if int(e.oldState()) == 4:
                        self.__topTitleBar.show()
        # catch full screen toggle event
        if obj.objectName() == 'mainWidget':
            if e.type() == 19:
                self.close()
            if e.type() == 105:
                self.__toggleFullScreenFromInnerWidget(e)

        # catch the enter event
        if e.type() == 10:
            self.unsetCursor()
            self._resizing = False
        # catch the title change event
        if e.type() == 33:
            self.__titleLbl.setText(obj.windowTitle())
        if obj.objectName() == 'navWidget':
            # catch the menubar double click or mouse move event
            if isinstance(obj, QMenuBar):
                if e.type() == 4 or e.type() == 5:
                    self.__execMenuBarMoveOrDoubleClickEvent(e)
                elif e.type() == 100:
                    cornerWidget = self.getCornerWidget()
                    if cornerWidget:
                        # apply background color change to corner widget and title label
                        color = self.__menubar.palette().color(QPalette.Base)
                        cornerWidget.setStyleSheet(f'QWidget {{ background-color: {color.name()} }};')
                        self.__titleLbl.setStyleSheet(f'QWidget {{ background-color: {color.name()} }};')

            # catch the titlebar double click or mouse move event
            elif isinstance(obj, TopTitleBarWidget):
                if e.type() == 4 or e.type() == 5:
                    self.__execTitleBarMoveOrDoubleClickEvent(e)
        return super().eventFilter(obj, e)

    def __toggleFullScreenFromInnerWidget(self, e):
        inner_state = int(e.oldState())
        if inner_state == 0 or inner_state == 4:
            if inner_state == 0:
                if isinstance(self.__topTitleBar, TopTitleBarWidget):
                    self.__topTitleBar.hide()
                self.showFullScreen()
            else:
                if isinstance(self.__topTitleBar, TopTitleBarWidget):
                    self.__topTitleBar.show()
                self.showNormal()
            title_bar_state = self.windowState()
            self.__widget.setWindowState(title_bar_state)

    def __execMenuBarMoveOrDoubleClickEvent(self, e):
        p = e.pos()
        if self.__menubar.actionAt(p):
            pass
        else:
            if self.__menubar.activeAction():
                pass
            else:
                # double click (show maximized/normal)
                if e.type() == 4 and e.button() == Qt.LeftButton:
                    self.__showNormalOrMaximized()
                # move
                else:
                    self._move()

    def __execTitleBarMoveOrDoubleClickEvent(self, e):
        if e.type() == 4 and e.button() == Qt.LeftButton:
            self.__showNormalOrMaximized()
        else:
            self._move()

    def __showNormalOrMaximized(self):
        self.__execShowNormalOrMaximized()

    def __execShowNormalOrMaximized(self):
        if self.isMaximized():
            self.showNormal()
        else:
            self.showMaximized()

    def setButtonHint(self, hint):
        self.__btnHint = hint

    def __getProperButtonsWidget(self, widget, btnWidget=None):
        if btnWidget:
            pass
        else:
            if self.__style == 'windows':
                btnWidget = WindowsCornerWidget(widget, self.__btnHint)
            elif self.__style == 'mac':
                btnWidget = MacCornerWidget(widget, self.__btnHint)
        return btnWidget

    # btnWidget(user-customized button widget), currently being developed
    def setButtons(self, btnWidget=None, align=Qt.AlignRight):
        # If window has a TopTitleBarWidget
        if isinstance(self.__topTitleBar, TopTitleBarWidget):
            self.__btnWidget = self.__getProperButtonsWidget(self.__topTitleBar, btnWidget)

            if isinstance(self.__btnWidget, WindowsCornerWidget):
                align = Qt.AlignRight
            elif isinstance(self.__btnWidget, MacCornerWidget):
                align = Qt.AlignLeft

            self.__topTitleBar.setCornerWidget(self.__btnWidget, align)
            iconTitleWidget = self.__topTitleBar.getIconTitleWidget()
            self.initTitleEvent(iconTitleWidget)
            self.initButtonsEvent()
        # If window has only a menu bar
        else:
            lay = QHBoxLayout()
            lay.setContentsMargins(0, 0, 0, 0)
            self.__btnWidget = self.__getProperButtonsWidget(self.__menubar, btnWidget)
            self.initButtonsEvent()
            lay.addWidget(self.__btnWidget)

            w = h = self.__titleLbl.font().pointSize() * 3 * qApp.screens()[0].logicalDotsPerInch()/96.0
            self.__btnWidget.setButtonSize(w, h)

            cornerWidget = QWidget()
            cornerWidget.setLayout(lay)
            # set the corner widget that already exists in QMenuBar
            if self.__menubar:
                existingCornerWidget = self.__menubar.cornerWidget(Qt.TopRightCorner)
                if existingCornerWidget:
                    lay.insertWidget(0, existingCornerWidget)
                    if self.__titleLbl.text():
                        lay.insertWidget(0, self.__titleLbl)

                cornerWidget.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.MinimumExpanding)
                cornerWidget.setMinimumHeight(self.__menubar.height())

                self.__menubar.setCornerWidget(cornerWidget, Qt.TopRightCorner)

    def __getWindowTitle(self, title):
        if title:
            pass
        else:
            title = self.__widget.windowTitle()
        return title

    def __getWindowIcon(self, icon_filename):
        if icon_filename:
            pass
        else:
            icon_filename = self.__widget.windowIcon().name()
        return icon_filename

    def __setWindowIcon(self, icon_filename):
        icon_filename = self.__getWindowIcon(icon_filename)
        self.setWindowIcon(QIcon(icon_filename))

    def setMenuTitle(self, title: str = '', icon_filename: str = '', font: QFont = QFont('Arial', 9)):
        # set menu title
        title = self.__getWindowTitle(title)
        self.__menubar = MenuBar()
        self.__menubar.setTitle(title, font)
        self.__menubar.setIcon(icon_filename)
        self.__titleLbl = self.__menubar.getTitle()
        self.__iconLbl = self.__menubar.getIcon()
        self.setWindowTitle(title)
        self.__setWindowIcon(icon_filename)

    def setTopTitleBar(self, title: str = '', icon_filename: str = '', font: QFont = QFont('Arial', 14),
                       align=Qt.AlignCenter, bottom_separator: bool = False):
        title = self.__getWindowTitle(title)
        self.__setWindowIcon(icon_filename)

        if isinstance(self.__menubar, QMenuBar):
            self.__topTitleBar = TopTitleBarWidget(self.__menubar, text=title, font=font, icon_filename=icon_filename,
                                                   align=align)
            self.__menubar.removeEventFilter(self)
        else:
            self.__topTitleBar = TopTitleBarWidget(self.__widget, text=title, font=font, icon_filename=icon_filename,
                                                   align=align)
        self.__topTitleBar.installEventFilter(self)
        self.__topTitleBar.setObjectName('navWidget')
        if bottom_separator:
            self.__topTitleBar.setBottomSeparator()

        lay = self.layout()
        centralWidget = lay.itemAt(0).widget()
        lay.addWidget(self.__topTitleBar, 0, 0, 1, 1)
        lay.addWidget(centralWidget, 1, 0, 1, 1)

        self.setPressToMove(False)

        # Set app title
        self.setWindowTitle(title)

    def getCornerWidget(self):
        return self.__menubar.cornerWidget()

    def initTitleEvent(self, titleWidget):
        self.__windowTitleIconLabel = titleWidget.getSvgLabel()
        self.__titleLbl = titleWidget.getTextLabel()

    def initButtonsEvent(self):
        self.__minimizeBtn = self.__btnWidget.getMinimizedBtn()
        self.__maximizeBtn = self.__btnWidget.getMaximizedBtn()
        self.__closeBtn = self.__btnWidget.getCloseBtn()

        self.__minimizeBtn.clicked.connect(self.showMinimized)
        self.__maximizeBtn.clicked.connect(self.__showNormalOrMaximized)
        self.__closeBtn.clicked.connect(self.close)

    def getInnerWidget(self):
        return self.__widget